/***************************************************************************//**
 * @file
 * @brief Top level application functions
 *******************************************************************************
 * # License
 * <b>Copyright 2020 Silicon Laboratories Inc. www.silabs.com</b>
 *******************************************************************************
 *
 * The licensor of this software is Silicon Laboratories Inc. Your use of this
 * software is governed by the terms of Silicon Labs Master Software License
 * Agreement (MSLA) available at
 * www.silabs.com/about-us/legal/master-software-license-agreement. This
 * software is distributed to you in Source Code format and is governed by the
 * sections of the MSLA applicable to Source Code.
 *
 ******************************************************************************/


/***************************************************************************//**
 * @file     app.c
 * @brief    EFM32 Segment LCD Display driver extension demonstration
 *
 * @details  This application demonstrates the usage of the Segment LCD driver
 *           extension by individually turning on and off the segments of the
 *           lower (7 alphanumeric characters) and the upper (4 seven-segment
 *           digits) displays.
 *
 * Created on:  2023. may. 06.
 *
 * Updated on: 2024. Sept. 20. (Orosz Gyorgy <orosz@mit.bme.hu>)
 *                - touch slider added
 *
 * Author:      NASZALY Gabor <naszaly@mit.bme.hu>
 ******************************************************************************/


/***************************************************************************//**
 * Includes
 ******************************************************************************/

/*
 * Header for the SegmentLCD driver extension
 */
#include "segmentlcd_individual.h"

/*
 * "segmentlcd.h" is also required, as the SegmentLCD driver extension does not
 * provide any initialization functions. We use the base SegmentLCD driver to
 * to initialize the display.
 */
#include "segmentlcd.h"

/*
 * "sl_udelay.h" is used only by the demo functions to slow things down.
 * Otherwise it is not required to use the SegmentLCD driver extension.
 */
#include <sl_udelay.h>

/*
 * Header for the capacitive sensor
 */
#include "caplesense.h"

/*
 * Header for device peripheral description
 */
#include "em_device.h"
#include "em_cmu.h"

/***************************************************************************//**
 * Globals
 ******************************************************************************/

/*
 * SegmentLCD_UpperSegments() and SegmentLCD_LowerSegments() are used to update
 * the display with new data. Display data is expected by these functions to be
 * located in arrays with SegmentLCD_UpperCharSegments_TypeDef and
 * SegmentLCD_LowerCharSegments_TypeDef elements, respectively.
 */
SegmentLCD_UpperCharSegments_TypeDef upperCharSegments[SEGMENT_LCD_NUM_OF_UPPER_CHARS];
SegmentLCD_LowerCharSegments_TypeDef lowerCharSegments[SEGMENT_LCD_NUM_OF_LOWER_CHARS];


volatile uint32_t msTicks; /* counts 1ms timeTicks */
int sliderPos;
int sliderDownsc = 0, sliderDownscOld = 0;

/***************************************************************************//**
 * Function definitions
 ******************************************************************************/


/***************************************************************************//**
 * @brief SysTick_Handler
 *   Interrupt Service Routine for system tick counter
 * @note
 *   No wrap around protection
 ******************************************************************************/
void SysTick_Handler(void)
{
  msTicks++;       /* increment counter necessary in Delay()*/
}

/***************************************************************************//**
 * @brief Delays number of msTick Systicks (typically 1 ms)
 * @param dlyTicks Number of ticks to delay
 ******************************************************************************/
void Delay(uint32_t dlyTicks)
{
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < dlyTicks) ;
}


/***************************************************************************//**
 * @brief Demo function for upper segments
 ******************************************************************************/
void demoUpperSegments(void);
/***************************************************************************//**
 * @brief Demo function for lower segments
 ******************************************************************************/
void demoLowerSegments(void);



/***************************************************************************//**
 * Initialize application.
 ******************************************************************************/
void app_init(void)
{
  /*
   * The SegmentLCD driver extension does not provide an initialization
   * functions. However, we need to call the initialization function of the
   * underlying base SegmentLCD driver.
   */
  SegmentLCD_Init(false);

  /* Enable LCD without voltage boost */
  SegmentLCD_Init(false);

  CAPLESENSE_Init(false);

  /* Setup SysTick Timer for 1 msec interrupts  */
  if (SysTick_Config(CMU_ClockFreqGet(cmuClock_CORE)/1000)) {
    while (1) ;
  }

  demoLowerSegments();
  demoUpperSegments();


}


/***************************************************************************//**
 * App ticking function.
 ******************************************************************************/
void app_process_action(void)
{


  // delete segments
  for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_LOWER_CHARS; p++) {
     for (uint8_t s = 0; s < 15; s++) {
        lowerCharSegments[p].raw = 1 << s;
     }
  }


  // get slider position
  sliderPos = CAPLESENSE_getSliderPosition();

  // display slider position
  SegmentLCD_Number(sliderPos);

  // calculate position
  sliderDownscOld = sliderDownsc;
  sliderDownsc = sliderPos/8;

  // set segment lines belonging to slider
  lowerCharSegments[sliderDownsc].j = 1;
  lowerCharSegments[sliderDownsc].p = 1;
  lowerCharSegments[sliderDownsc].d = 1;

  // draw LCD
  SegmentLCD_LowerSegments(lowerCharSegments);
  //Delay(10);

}




/*
 * This function demonstrates the usage of the extension driver for the upper
 * part of the LCD.
 */
void demoUpperSegments() {
   // Turn on all segments one-by-one
   // Only one segment is turned on at any given time
   // Using 7 bit binary (raw) value
   for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_UPPER_CHARS; p++) {
      for (uint8_t s = 0; s < 8; s++) {
         upperCharSegments[p].raw = 1 << s;
         SegmentLCD_UpperSegments(upperCharSegments);
         sl_udelay_wait(100000);
      }
   }

   // Turn on all segments one-by-one
   // All the previous segments are left turned on
   // Using dedicated (bit field) values
   for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_UPPER_CHARS; p++) {
      upperCharSegments[p].a = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);

      upperCharSegments[p].b = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);

      upperCharSegments[p].c = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);

      upperCharSegments[p].d = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);

      upperCharSegments[p].e = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);

      upperCharSegments[p].f = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);

      upperCharSegments[p].g = 1;
      SegmentLCD_UpperSegments(upperCharSegments);
      sl_udelay_wait(100000);
   }

   // Clear all segments
   for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_UPPER_CHARS; p++) {
      upperCharSegments[p].raw = 0;
      SegmentLCD_UpperSegments(upperCharSegments);
   }
}

/*
 * This function demonstrates the usage of the extension driver for the lower
 * part of the LCD.
 */
void demoLowerSegments() {
   // Turn on all segments one-by-one
   // Only one segment is turned on at any given time
   // Using 14 bit binary (raw) value
   for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_LOWER_CHARS; p++) {
      for (uint8_t s = 0; s < 15; s++) {
         lowerCharSegments[p].raw = 1 << s;
         SegmentLCD_LowerSegments(lowerCharSegments);
         sl_udelay_wait(100000);
      }
   }

   // Turn on all segments one-by-one
   // All the previous segments are left turned on
   // Using dedicated (bit field) values
   for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_LOWER_CHARS; p++) {
      lowerCharSegments[p].a = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].b = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].c = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].d = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].e = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].f = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].g = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].h = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].j = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].k = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].m = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].n = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].p = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);

      lowerCharSegments[p].q = 1;
      SegmentLCD_LowerSegments(lowerCharSegments);
      sl_udelay_wait(100000);
   }

   // Clear all segments
   for (uint8_t p = 0; p < SEGMENT_LCD_NUM_OF_LOWER_CHARS; p++) {
      lowerCharSegments[p].raw = 0;
      SegmentLCD_LowerSegments(lowerCharSegments);
   }
}


